'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var showLog = function () {
    var _ref4 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee4(info) {
        var url, data, log, logs;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
            while (1) {
                switch (_context4.prev = _context4.next) {
                    case 0:
                        url = info.logUrl + '?offset=' + info.lastLines;
                        _context4.next = 3;
                        return request(url);

                    case 3:
                        data = _context4.sent;

                        if (!(!data || !data.log)) {
                            _context4.next = 11;
                            break;
                        }

                        _context4.next = 7;
                        return helper.delay(2000);

                    case 7:
                        _context4.next = 9;
                        return showLog(info);

                    case 9:
                        _context4.next = 21;
                        break;

                    case 11:
                        log = data.log.trim();
                        logs = log.split('\n');

                        info.lastLines += logs.length;
                        logs = logs.filter(function (log) {
                            return !/^(Success|Fail)/.test(log);
                        });
                        console.log(logs.join('\n'));

                        if (~log.indexOf('[done] Sync ' + info.name)) {
                            _context4.next = 21;
                            break;
                        }

                        _context4.next = 19;
                        return helper.delay(2000);

                    case 19:
                        _context4.next = 21;
                        return showLog(info);

                    case 21:
                    case 'end':
                        return _context4.stop();
                }
            }
        }, _callee4, this);
    }));

    return function showLog(_x6) {
        return _ref4.apply(this, arguments);
    };
}();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var fs = require('fs');
var shell = require('./shell');
var helper = require('./helper');
var request = require('./request');
var configs = require('../configs');

/**
 * 获取某个包的仓库版本
 * @param {string} name
 * @param {string} tag
 * @returns {Promise<*>}
 */
exports.getInfo = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
    var name = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : configs.pkgName;
    var tag = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'latest';
    var registry;
    return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
            switch (_context.prev = _context.next) {
                case 0:
                    registry = helper.isInternal(name) ? configs.registry : configs.sourceRegistry;
                    _context.prev = 1;
                    _context.next = 4;
                    return request(`${registry}/${name}/${tag}`);

                case 4:
                    return _context.abrupt('return', _context.sent);

                case 7:
                    _context.prev = 7;
                    _context.t0 = _context['catch'](1);
                    return _context.abrupt('return', {});

                case 10:
                case 'end':
                    return _context.stop();
            }
        }
    }, _callee, this, [[1, 7]]);
}));

/**
 * 获取某个包的最新版本
 * @param name
 * @returns {Promise<string>}
 */
exports.getLatest = function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee2(name) {
        var pkg;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        _context2.next = 2;
                        return this.getInfo(name);

                    case 2:
                        pkg = _context2.sent;
                        return _context2.abrupt('return', pkg.version);

                    case 4:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, this);
    }));

    return function (_x3) {
        return _ref2.apply(this, arguments);
    };
}();

/**
 * 返回某个包的最新版本
 * @param {string} pkg
 * @return {string}
 */
exports.getLatestSync = function (pkg) {
    return shell.exec('npm view ' + pkg + ' version --registry ' + configs.registry, 1);
};

exports.DEP_TYPE = {
    dependencies: 'prod',
    devDependencies: 'dev',
    optionalDependencies: 'optional',
    peerDependencies: 'peer'
};

/**
 * 获取依赖
 * @param {string|string[]} [part] - dependencies, devDependencies, optionalDependencies, peerDependencies[, allDependencies], //bundledDependencies//
 * @param {boolean} [json=false] - 是否返回json
 * @returns {Array}
 */
exports.getDependencies = function (part, json) {
    var _this = this;

    try {
        var pkg = JSON.parse(fs.readFileSync('package.json', 'utf8'));

        var deps = [];
        var maps = {};
        var parts = [];

        if (!part) {
            parts = 'dependencies';
        } else if (part === 'all') {
            parts = Object.keys(this.DEP_TYPE);
        } else if (typeof part === 'string') {
            parts = [].slice.call(arguments);
        } else if (Array.isArray(part)) {
            parts = part;
        }

        parts.forEach(function (part) {
            var depType = _this.DEP_TYPE[part];
            var dependencies = pkg[part];
            if (depType && dependencies && typeof dependencies === 'object') {
                for (var p in dependencies) {
                    if (!dependencies.hasOwnProperty(p)) continue;
                    deps.push(maps[p] = {
                        name: p,
                        type: depType,
                        version: dependencies[p],
                        position: part
                    });
                }
            }
        });

        return json ? maps : deps;
    } catch (e) {
        return json ? {} : [];
    }
};

exports.hasPackageJSON = function () {
    return fs.existsSync('package.json');
};

exports.hasNpmLock = function () {
    return fs.existsSync('package-lock.json');
};

exports.hasYarnLock = function () {
    return fs.existsSync('yarn.lock');
};

exports.delNpmLock = function () {
    try {
        fs.unlinkSync('package-lock.json');
    } catch (e) {}
};

exports.delYarnLock = function () {
    try {
        fs.unlinkSync('yarn.lock');
    } catch (e) {}
};

exports.read = function () {
    return JSON.parse(fs.readFileSync('package.json', 'utf8'));
};

exports.save = function (pkg) {
    var text = fs.readFileSync('package.json', 'utf8');
    var tabs = /\s{4}"name"/.test(text) ? 4 : 2;
    fs.writeFileSync('package.json', JSON.stringify(pkg, null, tabs));
};

exports.sync = function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee3(packages, options) {
        var result, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, name, url, data, info;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        result = {
                            success: [],
                            failure: []
                        };
                        _iteratorNormalCompletion = true;
                        _didIteratorError = false;
                        _iteratorError = undefined;
                        _context3.prev = 4;
                        _iterator = packages[Symbol.iterator]();

                    case 6:
                        if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                            _context3.next = 28;
                            break;
                        }

                        name = _step.value;
                        url = `${configs.registry}/${name}/sync`;
                        _context3.prev = 9;
                        _context3.next = 12;
                        return request.put(url, {
                            params: {
                                publish: options.syncPublish,
                                nodeps: !options.dependencies
                            }
                        });

                    case 12:
                        data = _context3.sent;

                        if (!(!data || !data.ok)) {
                            _context3.next = 16;
                            break;
                        }

                        result.failure.push(name);
                        return _context3.abrupt('continue', 25);

                    case 16:
                        info = {
                            name: name,
                            lastLines: 0,
                            logUrl: `${configs.registry}/${name}/sync/log/${data.logId}`
                        };
                        _context3.next = 19;
                        return showLog(info);

                    case 19:
                        result.success.push(name);
                        _context3.next = 25;
                        break;

                    case 22:
                        _context3.prev = 22;
                        _context3.t0 = _context3['catch'](9);

                        result.failure.push(name);

                    case 25:
                        _iteratorNormalCompletion = true;
                        _context3.next = 6;
                        break;

                    case 28:
                        _context3.next = 34;
                        break;

                    case 30:
                        _context3.prev = 30;
                        _context3.t1 = _context3['catch'](4);
                        _didIteratorError = true;
                        _iteratorError = _context3.t1;

                    case 34:
                        _context3.prev = 34;
                        _context3.prev = 35;

                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }

                    case 37:
                        _context3.prev = 37;

                        if (!_didIteratorError) {
                            _context3.next = 40;
                            break;
                        }

                        throw _iteratorError;

                    case 40:
                        return _context3.finish(37);

                    case 41:
                        return _context3.finish(34);

                    case 42:
                        return _context3.abrupt('return', result);

                    case 43:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this, [[4, 30, 34, 42], [9, 22], [35,, 37, 41]]);
    }));

    return function (_x4, _x5) {
        return _ref3.apply(this, arguments);
    };
}();